import defaultLogo from "./logo.svg";
import "./App.scss";

import React,{useState} from "react"

import { Provider, connect } from "react-redux";
import { createStore, combineReducers, applyMiddleware } from "redux";
import thunk from "redux-thunk";

const jwtDecode = (token) => {
    try {
        let payload = JSON.parse(atob(token.split(".")[1]));

        return payload;
    } catch (e) {
        console.log(e);
    }
};

function cartReducer(state = {}, { type, good, count = 1 }) {
    if (count <= 0) {
        type = "CART_DELETE";
    }

    if (type === "CART_ADD") {
        return {
            ...state,
            [good["_id"]]: {
                good,
                count: good["_id"] in state ? state[good._id].count + count : count,
            },
        };
    }
    if (type === "CART_CHANGE") {
        return {
            ...state,
            [good["_id"]]: {
                good,
                count: count,
            },
        };
    }
    if (type === "CART_DELETE") {
        let { [good._id]: toRemove, ...newState } = state;
        return newState;
    }
    if (type === "CART_CLEAR") {
        return {};
    }
    return state;
}
function promiseReducer(state = {}, { type, name, status, payload, error }) {
    if (type === "PROMISE") {
        return {
            ...state,
            [name]: { status, payload, error },
        };
    }
    return state;
}
function authReducer(state, { type, token }) {
    if (state === undefined) {
        if (localStorage.authToken) {
            token = localStorage.authToken;
            type = "AUTH_LOGIN";
            state = {};
        }
    }

    if (type === "AUTH_LOGIN") {
        if (!token || !jwtDecode(token)) return {};
        localStorage.authToken = token;
        return {
            ...state,
            token: token,
            payload: jwtDecode(token),
        };
    }

    if (type === "AUTH_LOGOUT") {
        localStorage.removeItem("authToken");
        return {};
    }
    return state || {};
}
const getGQL = (url) => (query, variables) =>
    fetch(url, {
        method: "POST",
        headers: {
            "Content-Type": "application/json",
            ...(localStorage.authToken ? { Authorization: "Bearer " + localStorage.authToken } : {}),
        },
        body: JSON.stringify({ query, variables }),
    })
        .then((res) => res.json())
        .then((data) => {
            if (data.errors) {
                throw new Error(JSON.stringify(data.errors));
            } else return Object.values(data.data)[0];
        });

const backendURL = "http://shop-roles.asmer.fs.a-level.com.ua";
const gql = getGQL(backendURL + "/graphql");

const store = createStore(
    combineReducers({ auth: authReducer, promise: promiseReducer, cart: cartReducer }),
    applyMiddleware(thunk)
);

const actionPending = (name) => ({ type: "PROMISE", name, status: "PENDING" });
const actionFulfilled = (name, payload) => ({ type: "PROMISE", name, status: "FULFILLED", payload });
const actionRejected = (name, error) => ({ type: "PROMISE", name, status: "REJECTED", error });
const actionPromise = (name, promise) => async (dispatch) => {
    dispatch(actionPending(name));
    try {
        let payload = await promise;
        dispatch(actionFulfilled(name, payload));
        return payload;
    } catch (error) {
        dispatch(actionRejected(name, error));
    }
};
const actionRootCats = () =>
    actionPromise(
        "rootCats",
        gql(`query {
        CategoryFind(query: "[{\\"parent\\":null}]"){
            _id name
        }
    }`)
    );
const actionGoodById = (_id) =>
    actionPromise(
        "good",
        gql(
            `query GoodById($q: String){
            GoodFindOne(query: $q){
                _id name  price images{
                    url
                }
            }
        }`,
            { q: JSON.stringify([{ _id }]) }
        )
    );
const actionCatById = (
    _id //добавить подкатегории
) =>
    actionPromise(
        "catById",
        gql(
            `query catById($q: String){
        CategoryFindOne(query: $q){
            _id name goods {
                _id name price images {
                    url
                }
            }
        }
    }`,
            { q: JSON.stringify([{ _id }]) }
        )
    );
const actionOrders = () => (dispatch) =>
    dispatch(
        actionPromise(
            "orders",
            gql(`
            query orders{
                OrderFind(query:"[{}]"){
                    _id total createdAt orderGoods{
                        _id count price good{
                            name _id price images{
                                url
                            }
                        }
                    }
                }
            }
          `)
        )
    );
const actionNewOrder =
    (orderGoods = []) =>
    async (dispatch, getState) => {
        const { auth: token } = getState();
        if (!orderGoods.length) {
            return;
        }
        await dispatch(
            actionPromise(
                "newOrder",
                gql(
                    `mutation newOrder($order:OrderInput){
                OrderUpsert(order:$order){
                  _id total
                }
              }
              `,
                    {
                        order: {
                            orderGoods: orderGoods.map((orderGood) => ({
                                count: orderGood.count,
                                good: { _id: orderGood.good._id },
                            })),
                        },
                    }
                )
            )
        );
        let {
            promise: { newOrder },
        } = getState();
        if (newOrder.status === "FULFILLED") {
            dispatch(actionCartClear());
            dispatch(actionOrders(token));
        }
    };
const actionAuthLogin = (token) => ({
    type: "AUTH_LOGIN",
    token: token,
});
const actionLogin = (login, password) => async (dispatch) => {
    const token = await dispatch(
        actionPromise(
            "login",
            gql(
                `query log($login:String,$password:String){
                            login(login:$login,password:$password)
                        }
                        `,
                {
                    login: login,
                    password: password,
                }
            )
        )
    );
    await dispatch(actionAuthLogin(token));
    window.location = "#/";
};
const actionAuthLogout = () => ({ type: "AUTH_LOGOUT" });
const actionRegister = (login, password) => async (dispatch) => {
    await dispatch(
        actionPromise(
            "register",
            gql(
                `mutation register($login:String,$password:String){
                               UserUpsert(user:{login:$login,password:$password}){
                                   _id login
                               }
                           }`,
                {
                    login: login,
                    password: password,
                }
            )
        )
    );
};
const actionCartAdd = (good, count = 1) => ({ type: "CART_ADD", good, count:+count });
const actionCartChange = (good, count = 1) => ({ type: "CART_CHANGE", good, count:+count });
const actionCartDelete = (good) => ({ type: "CART_DELETE", good });
const actionCartClear = () => ({ type: "CART_CLEAR" });
const actionPageStart = () => async (dispatch, getState) => {
    const {
        auth: { token },
    } = store.getState();
    store.dispatch(actionCatById(token));
    store.dispatch(actionOrders(token));
};

const defaultCategories = [
    {
        _id: "5dc458985df9d670df48cc47",
        name: "     Smartphones",
    },
    {
        _id: "5dc4b2553f23b553bf354101",
        name: "Крупная бытовая техника",
    },
    {
        _id: "5dcac1b56d09c45440d14cf8",
        name: " Макароны",
    },
    {
        _id: "5dcac6cf6d09c45440d14cfd",
        name: "Drinks",
    },
    {
        _id: "5dcacaeb6d09c45440d14d04",
        name: "Салаты",
    },
    {
        _id: "61715b92ef4e1b3e3b67703c",
        name: "Приятный бонус",
    },
    {
        _id: "61e725dcf9be102f49b2c6c4",
        name: "Товарчики",
    },
    {
        _id: "5dc94bd00e36db246e3049ee",
        name: "Пицца",
    },
    {
        _id: "5dcabeeb6d09c45440d14cf6",
        name: "Макароны",
    },
];

const Logo = ({ logo = defaultLogo }) => <img src={logo} className="Logo" />;
const CategoryItem = ({ category: { _id, name } = {}, handleClick }) => (
    <li className="CategoryItem" onClick={() => handleClick(_id)}>
        {name}
    </li>
);

const RootCategories = ({ categories = defaultCategories, handleClickOnItem }) => (
    <ul className="RootCategories">
        {categories.map((category) => (
            <CategoryItem category={category} handleClick={handleClickOnItem} />
        ))}
    </ul>
);
const CRootCategories = connect((state) => ({ categories: state.promise?.rootCats?.payload || [] }), {
    handleClickOnItem: (_id) => actionCatById(_id),
})(RootCategories);

const GoodCard = ({ good: { name, images, price, _id }, token, handleOnClick } = {}) => {
    return (
        <div className="GoodCard row">
            <img src={`${backendURL}/${images ? images[0].url : ""}`} className="GoodImage col-6" alt="" />
            <div className="col-6">
                <div>
                    <strong>Name : </strong> {name}
                </div>
                <div>
                    <strong>Price : </strong> {price}
                </div>
                {token ? (
                    <div>
                        <button
                            onClick={() => handleOnClick({ name, images, price, _id })}
                            className="btn btn-success px-3"
                        >
                            Buy
                        </button>
                    </div>
                ) : (
                    false
                )}
            </div>
        </div>
    );
};
const CGoodCard = connect((state) => ({ token: state.auth?.token || false }), {
    handleOnClick: (good) => actionCartAdd(good),
})(GoodCard);

const GoodsList = ({ goods,category:{name:categoryName} } = {}) => (

    <div className="GoodList">
        <div className = "row text-center"><h5>{categoryName}</h5></div>
        <div className="row">
        {goods.map((good) => (
            <div className="col-12 col-md-6">
                <CGoodCard good={good} />
            </div>
        ))}
        </div>
    </div>
)
const CGoodsList = connect((state) => ({ goods: state.promise?.catById?.payload?.goods || [],category: state.promise?.catById?.payload || "" }))(GoodsList);

const CartItem = ({
    order: {
        count,
        good: { _id, name, images, price },
    },
    handleDelete,
    handleChange,
}) => (
    <div className="CartItem row mt-3">
        <img src={`${backendURL}/${images ? images[0].url : ""}`} className="GoodImage col-3" alt="" />
        <div className="col-5">
            <div>
                <strong>Name : </strong> {name}
            </div>
            <div>
                <strong>Price : </strong> {price} * {count} = {+price * +count}
            </div>
        </div>
        <div className="col-4">
            <div className=" w-100 d-flex justify-content-between flex-wrap ">
                <div className="d-flex">
                    <button
                        className="btn btn-info"
                        onClick={() => handleChange({ _id, name, images, price }, --count)}
                    >
                        -
                    </button>
                    <input className="countInput" type="number" value = {count} onChange ={(e)=>handleChange({ _id, name, images, price },e.target.value)}/>
                    <button
                        className="btn btn-info"
                        onClick={() => handleChange({ _id, name, images, price }, ++count)}
                    >
                        +
                    </button>
                </div>

                <button className="btn btn-danger " onClick={() => handleDelete({ _id, name, images, price })}>
                    Delete
                </button>
            </div>
        </div>
    </div>
);
const CCartItem = connect(() => ({}), {
    handleDelete: (good) => actionCartDelete(good),
    handleChange: (good, count) => actionCartChange(good, count),
})(CartItem);

const Cart = ({ cart, token, handleSubmit }) => {
    let cartSum = 0;
    Object.entries(cart).map(([_id, order]) => (cartSum += +order?.count * +order?.good?.price));

    return token && Object.keys(cart).length > 0 ? (
        <div className="Cart ">
            <div className="row header">Cart</div>
            <div className="row">
                <div className="col-12">
                    {Object.entries(cart).map(([_id, order]) => (
                        <CCartItem order={order} />
                    ))}
                </div>
            </div>
            <div className="row d-flex justify-content-between footer">
                <strong>Sum: {cartSum}</strong>
                <button className="btn btn-success" onClick={() => handleSubmit([...Object.values(cart)])}>
                    Submit
                </button>
            </div>
        </div>
    ) : (
        false
    );
};
const CCart = connect((state) => ({ cart: state.cart || [], token: state.auth.token }), {
    handleSubmit: (orderGoods = []) => actionNewOrder(orderGoods),
})(Cart);

const DashboardOrderGood = ({ orderGood: { count, price: total, good } } = {}) => {
    const { name, _id, price, images } = good || {};
    return (
        <div className="DashboardOrderGood row">
            <img src={`${backendURL}/${images ? images[0].url : ""}`} className="GoodImage col-3" alt="" />
            <div className="col-5">
                <div>
                    <strong>Name : </strong> {name}
                </div>
                <div>
                    <strong>Price : </strong> {price} * {count} = {+price * +count}
                </div>
            </div>
            <div className="col-4">
                <div className=" w-100 d-flex justify-content-between">
                    <div className="d-flex"></div>
                </div>
            </div>
        </div>
    );
};

const DashboardOrder = ({ order: { _id, createdAt, total, orderGoods } } = {}) => (
    <div className="DashboardOrder ">
        <div className="row header">
            <h5>Date: {new Date(+createdAt).toDateString()}</h5> <strong>total: {total}</strong>
        </div>

        {orderGoods?.map((orderGood) => (
            <DashboardOrderGood orderGood={orderGood} />
        ))}
    </div>
);
const Dashboard = ({ orders, token }) =>
    token && Object.keys(orders).length > 0 ? (
        <div className="Dashboard ">
            <div className="row header">Orders</div>
            {orders.map((order) => (
                <DashboardOrder order={order || {}} />
            ))}
        </div>
    ) : (
        false
    );

const CDashboard = connect((state) => ({ orders: state.promise.orders?.payload || [], token: state.auth?.token }))(
    Dashboard
);


const Input = ({}) => {
    const [text, setText] = useState("")
    return <input value = {text} onChange = {e=>setText(e.target.value)}/>
}


const LoginForm = ({onLogin}) => {
    const [login,setLogin] = useState("")
    const [password,setPassword] = useState("")
    return (
        <div className="d-flex">
            <input className="input" placeholder = "Login" value={login} onChange = {(e)=>setLogin(e.target.value)}/>
            <input type = "password"className="input" placeholder = "Password" value={password} onChange = {e=>setPassword(e.target.value)}/>
            <button disabled = {login.length<4 || password.length<4} className="btn btn-success" onClick={()=> onLogin(login,password)}>
                login
            </button>
        </div>
    );
};
const CLoginForm = connect((state) => ({}), { onLogin: ( login, password ) => actionLogin(login, password) })(
    LoginForm
);
const Logout = ({ handleOnClick, login }) => <button onClick={handleOnClick}>{`Logout (${login || ""})`}</button>;
const CLogout = connect((state) => ({ login: state.auth.payload?.sub?.login }), { handleOnClick: actionAuthLogout })(
    Logout
);
const AuthBlock = ({ token }) => <div className="d-flex">{token ? <CLogout /> : <CLoginForm />}</div>;
const CAuthBlock = connect((state) => ({ token: state.auth?.token }))(AuthBlock);

const Header = ({ children }) => (
    <header>
        <CAuthBlock />
        {children}
    </header>
);
const Aside = ({ children }) => (
    <aside className="Aside flex-grow-1  ">
        <CRootCategories />
        {children}
    </aside>
);

const Content = ({ children }) => <section className="Content  row">{children}</section>;

const Main = ({ children }) => (
    <main className="Main d-flex">
        <Aside />
        <Content>
            <CGoodsList />
            <CCart />
            <CDashboard />
        </Content>
        {children}
    </main>
);

const Footer = ({ children }) => <footer className="row">{children}</footer>;

function App() {
    return (
        <Provider store={store}>
            <div className="App  container">
                <Header />
                <Main />
                <Footer />
            </div>
        </Provider>
    );
}

store.subscribe(() => console.log(store.getState()));
store.dispatch(actionRootCats());
store.dispatch(actionPageStart());
export default App;
